/*
* Copyright (c) 2006-2009 Erin Catto http://www.box2d.org
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#ifndef IFORCE2D_TANK_H
#define IFORCE2D_TANK_H

#include <vector>
#include <set>
#include "../Framework/b2dJson.h"
#include "bitmap.h"

#include <stdio.h>

#define DEGTORAD 0.0174532925199432957f
#define RADTODEG 57.295779513082320876f

//this will be set as user data in fixtures
class ConveyorSettings {
protected:
    struct conveyorSegment {
        float32 minAngle;
        float32 maxAngle;
        float32 tangentialVelocity;
    };
    
    vector<conveyorSegment> m_conveyorSegments;
    
public:
    ConveyorSettings() {}
    
    void addSegment(float minAngle, float32 maxAngle, float32 tangentialVelocity) {
        conveyorSegment segment;
        segment.minAngle = minAngle;
        segment.maxAngle = maxAngle;
        segment.tangentialVelocity = tangentialVelocity;
        m_conveyorSegments.push_back(segment);
    }
    
    float32 getTangentialVelocity(b2Vec2 localNormal) {
        float32 angle = b2Atan2(localNormal.y, localNormal.x);
        for (int i = 0; i < m_conveyorSegments.size(); i++) {
            conveyorSegment& segment = m_conveyorSegments[i];
            //if ( angle >= segment.minAngle && angle < segment.maxAngle )
            return segment.tangentialVelocity;
        }
        return 0;
    }
};

struct splinePoint {
    b2Body* body;
    b2Vec2 localPoint;
    b2Vec2 getPointNow() { return body->GetWorldPoint(localPoint); }
};

struct textureParams {
    b2Body* body;
    b2Vec2 verts[4];
    char textureFile[64];
    GLuint texId;

    void draw() {
        glBindTexture(GL_TEXTURE_2D, texId);
        glPushMatrix();
        b2Vec2 pos = body->GetPosition();
        glTranslatef(pos.x, pos.y, 0);
        glRotatef(body->GetAngle()*RADTODEG, 0,0,1);

        glBegin(GL_TRIANGLE_FAN);
        glTexCoord2f(0,0);
        glVertex2f(verts[0].x, verts[0].y);
        glTexCoord2f(1,0);
        glVertex2f(verts[1].x, verts[1].y);
        glTexCoord2f(1,1);
        glVertex2f(verts[2].x, verts[2].y);
        glTexCoord2f(0,1);
        glVertex2f(verts[3].x, verts[3].y);
        glEnd();

        glBegin(GL_LINE_LOOP);
        glVertex2f(verts[0].x, verts[0].y);
        glVertex2f(verts[1].x, verts[1].y);
        glVertex2f(verts[2].x, verts[2].y);
        glVertex2f(verts[3].x, verts[3].y);
        glEnd();

        glPopMatrix();
    }
};

class TankDebugDraw : public DebugDraw
{
public:
    void addNoDrawBody(b2Body* b) {
        m_noDrawBodyList.insert(b);
    }

    void DrawPolygon(class b2Fixture* fixture, const b2Vec2* vertices, int32 vertexCount, const b2Color& color) {
        if ( !fixture || m_noDrawBodyList.find(fixture->GetBody()) == m_noDrawBodyList.end() )
            DebugDraw::DrawPolygon(fixture, vertices, vertexCount, color);
    }
    void DrawSolidPolygon(class b2Fixture* fixture, const b2Vec2* vertices, int32 vertexCount, const b2Color& color) {
        if ( !fixture || m_noDrawBodyList.find(fixture->GetBody()) == m_noDrawBodyList.end() )
            DebugDraw::DrawSolidPolygon(fixture, vertices, vertexCount, color);
    }
    void DrawCircle(class b2Fixture* fixture, const b2Vec2& center, float32 radius, const b2Color& color) {
        if ( !fixture || m_noDrawBodyList.find(fixture->GetBody()) == m_noDrawBodyList.end() )
            DebugDraw::DrawCircle(fixture, center, radius, color);
    }
    void DrawSolidCircle(class b2Fixture* fixture, const b2Vec2& center, float32 radius, const b2Vec2& axis, const b2Color& color) {
        if ( !fixture || m_noDrawBodyList.find(fixture->GetBody()) == m_noDrawBodyList.end() )
            DebugDraw::DrawSolidCircle(fixture, center, radius, axis, color);
    }

    set<b2Body*> m_noDrawBodyList;
};

class Tank : public Test
{
public:
    
    enum {
        k_left = 	0x01,
        k_right = 	0x02
    };

    enum {
        k_drawTextureShowTracks,
        k_drawTextureCoverTracks,
        k_drawFixtures
    };

    void extractTextureParameters(const char* textureName, b2dJson& json) {
        char fixtureName[128];
        sprintf(fixtureName, "img_%s", textureName);
        b2Fixture* f = json.getFixtureByName(fixtureName);
        if ( f ) {
            textureParams tp;
            tp.body = f->GetBody();
            sprintf(tp.textureFile, "%s.bmp", textureName);
            tp.texId = 0;
            b2PolygonShape* polygonShape = (b2PolygonShape*)f->GetShape();
            for (int i = 0; i < 4; i++) {
                tp.verts[i] = polygonShape->GetVertex(i);
                //printf("%d: %f, %f\n", i, tp.verts[i].x, tp.verts[i].y);
            }
            //printf("---\n");
            //fflush(stdout);
            m_tankDebugDraw->addNoDrawBody(f->GetBody());
            m_textureParams.push_back(tp);
            f->GetBody()->DestroyFixture(f);
        }
    }

    void loadTexture(const char* filename, GLuint& texId) {
        CBitmap bitmap;
        if ( bitmap.Load(filename)) {
            unsigned int size;
            bitmap.GetBits(NULL, size, 0,0,0,0);
            unsigned char* rgba = new unsigned char[bitmap.GetWidth()*bitmap.GetHeight()*size];
            bitmap.GetBits(rgba, size, 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000);
            glGenTextures(1, &texId);
            glBindTexture(GL_TEXTURE_2D, texId);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, bitmap.GetWidth(), bitmap.GetHeight(), 0, GL_RGBA, GL_UNSIGNED_BYTE, rgba);
            delete rgba;
        }
        else {
            printf("Could not load file: %s", filename);
            fflush(stdout);
        }
    }
    
    Tank()
    {
        m_tankDebugDraw = new TankDebugDraw();

        //load world from JSON file
        b2dJson json;
        b2World* world = json.readFromFile("tank.json");
        if ( world ) {
            //replace testbed's world
            delete m_world;
            m_world = world;
            
            //re-set standard testbed stuff
            m_world->SetContactListener(this);
            m_world->SetDebugDraw( m_tankDebugDraw );
            b2BodyDef bodyDef;
            m_groundBody = m_world->CreateBody(&bodyDef);//needed for testbed mousejoint
            
            json.getBodiesByName("wheelguide", m_wheelguides);
            for (int i = 0; i < m_wheelguides.size(); i++)
                m_tankDebugDraw->addNoDrawBody(m_wheelguides[i]);
            json.getBodiesByName("endwheel", m_endwheels);
            for (int i = 0; i < m_endwheels.size(); i++)
                m_tankDebugDraw->addNoDrawBody(m_endwheels[i]);
            
            vector<b2Body*> trackSegments;
            json.getBodiesByName("tracksegment", trackSegments);
            for (int i = 0; i < trackSegments.size(); i++) {
                trackSegments[i]->SetLinearVelocity( b2Vec2(0,0) );
                m_tankDebugDraw->addNoDrawBody(trackSegments[i]);
            }
            
            //spline points
            for (int i = 0; i < 100; i++ ) {
                char buf[32];
                sprintf(buf, "spline%d", i);
                b2Fixture* f = json.getFixtureByName(buf);
                if ( !f )
                    break;
                b2CircleShape* s = (b2CircleShape*)f->GetShape();
                splinePoint sp;
                sp.localPoint = s->m_p;
                sp.body = f->GetBody();
                m_splinePoints.push_back(sp);
                f->GetBody()->DestroyFixture(f);
            }
            
            //get named fixtures named "left" or "right" from the JSON info
            vector<b2Fixture*> leftConveyors;
            vector<b2Fixture*> rightConveyors;
            json.getFixturesByName("left", leftConveyors);
            json.getFixturesByName("right", rightConveyors);
            
            //set up the user data of the named fixtures
            for (int i = 0; i < leftConveyors.size(); i++) {
                ConveyorSettings* conveyorSettings = new ConveyorSettings();
                conveyorSettings->addSegment(10 * DEGTORAD, 170 * DEGTORAD, -1);
                leftConveyors[i]->SetUserData(conveyorSettings);
                m_conveyorSettingsToDelete.push_back(conveyorSettings);
            }
            for (int i = 0; i < rightConveyors.size(); i++) {
                ConveyorSettings* conveyorSettings = new ConveyorSettings();
                conveyorSettings->addSegment(0 * DEGTORAD, 360 * DEGTORAD, 1);
                rightConveyors[i]->SetUserData(conveyorSettings);
                m_conveyorSettingsToDelete.push_back(conveyorSettings);
            }
            
            m_tankBody = NULL;
            vector<b2Body*> tankbodies;
            if ( json.getBodiesByName("tankbody", tankbodies) ) {
                tankbodies[0]->SetSleepingAllowed(false);
                m_tankBody = tankbodies[0];
            }

            extractTextureParameters("drivertorso", json);
            extractTextureParameters("driverhead", json);
            extractTextureParameters("driverarmupper", json);
            extractTextureParameters("driverarmlower", json);

            extractTextureParameters("gunbarrel", json);
            extractTextureParameters("tankbody", json);

            extractTextureParameters("trackcover", json);
            
//            b2Fixture* f = json.getFixtureByName("img_driverhead");
//            if ( f ) {
//                textureParams tp;
//                tp.body = f->GetBody();
//                sprintf(tp.textureFile, "%s.bmp", "driverhead");
//                tp.texId = 0;
//                b2PolygonShape* polygonShape = (b2PolygonShape*)f->GetShape();
//                for (int i = 0; i < 4; i++) {
//                    tp.verts[i] = polygonShape->GetVertex(i);
//                    //printf("%d: %f, %f\n", i, tp.verts[i].x, tp.verts[i].y);
//                }
//                //printf("---\n");
//                //fflush(stdout);
//                m_tankDebugDraw->addNoDrawBody(f->GetBody());
//                m_textureParams.push_back(tp);
//                f->GetBody()->DestroyFixture(f);
//            }



        }
        
        m_maxSpeed = 5;
        m_currentSpeed = 0;
        m_wheelAngle = 0;
        for (int i = 0; i < 9; i++)
            m_wheelAngles[i] = RandomFloat(0, 2 * M_PI);
        
        m_firstTrackPointOffset = 0;
        
        m_keyState = 0;
        m_drawMode = k_drawTextureShowTracks;
        
        
        for (int i = 0; i < m_textureParams.size(); i++) {
            textureParams& tp = m_textureParams[i];
            CBitmap bitmap;
            if ( bitmap.Load(tp.textureFile)) {
                unsigned int size;
                bitmap.GetBits(NULL, size, 0,0,0,0);
                unsigned char* rgba = new unsigned char[bitmap.GetWidth()*bitmap.GetHeight()*size];
                //bitmap.GetBits(rgba, size, 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000);
                bitmap.GetBits(rgba, size, 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000);
                /*int w = bitmap.GetWidth();
                int h = bitmap.GetHeight();
                for (int p = 0; p < w*h; p++) {
                    if ( rgba[p*4+0] == 255 && rgba[p*4+1] == 0 && rgba[p*4+2] == 255 )
                        rgba[p*4+3] = 0;
                    int alpha = rgba[p*4+3];
                    int adsf = 0;
                    printf("%d ", alpha);
                    //else
                    //    rgba[p*4+3] = 255;//no idea why this is necessary, but without it the whole texture alpha seems less than 255
                }
                fflush(stdout);*/
                
                glGenTextures(1, &tp.texId);
                glBindTexture(GL_TEXTURE_2D, tp.texId);
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, bitmap.GetWidth(), bitmap.GetHeight(), 0, GL_RGBA, GL_UNSIGNED_BYTE, rgba);
                delete rgba;
            }
        }

        loadTexture("drivewheel.bmp", m_driveWheelTexId);
        loadTexture("rollerwheel.bmp", m_rollerWheelTexId);
        loadTexture("wheelshadow.bmp", m_wheelShadowTexId);
        loadTexture("tracksegment.bmp", m_trackSegmentTexId);
        
    }
    
    ~Tank()
    {
        for (int i = 0; i < m_textureParams.size(); i++) {
            textureParams& tp = m_textureParams[i];
            glDeleteTextures(1, &tp.texId);
        }
        glDeleteTextures(1, &m_driveWheelTexId);
        glDeleteTextures(1, &m_rollerWheelTexId);
        glDeleteTextures(1, &m_wheelShadowTexId);
        glDeleteTextures(1, &m_trackSegmentTexId);
        for (int i = 0; i < m_conveyorSettingsToDelete.size(); i++)
            delete m_conveyorSettingsToDelete[i];
        delete m_tankDebugDraw;
    }
    
    void Keyboard(unsigned char key)
    {
        switch (key)
        {
        case 'a': m_keyState |= k_left; break;
        case 'd': m_keyState |= k_right; break;
        case 't':
            m_drawMode = (m_drawMode+1)%3;
            if ( m_drawMode == k_drawFixtures )
                m_world->SetDebugDraw( &m_debugDraw );
            else
                m_world->SetDebugDraw( m_tankDebugDraw );
            break;
        }
    }
    
    void KeyboardUp(unsigned char key)
    {
        switch (key)
        {
        case 'a': m_keyState &= ~k_left; break;
        case 'd': m_keyState &= ~k_right; break;
        }
    }
    
    void PreSolve(b2Contact* contact, const b2Manifold* oldManifold)
    {
        b2Fixture* fixtureA = contact->GetFixtureA();
        b2Fixture* fixtureB = contact->GetFixtureB();
        if ( !fixtureA->GetUserData() && !fixtureB->GetUserData() )
            return;
        
        contact->SetTangentSpeed(m_currentSpeed);
        
        b2WorldManifold worldManifold;
        contact->GetWorldManifold(&worldManifold);
        
        float tangentialVelocity = 0;
        
        if ( fixtureA->GetUserData() ) {
            ConveyorSettings* conveyorSettings = (ConveyorSettings*)fixtureA->GetUserData();
            b2Vec2 localNormalA = fixtureA->GetBody()->GetLocalVector(worldManifold.normal);
            tangentialVelocity += conveyorSettings->getTangentialVelocity(localNormalA);
        }
        if ( fixtureB->GetUserData() ) {
            ConveyorSettings* conveyorSettings = (ConveyorSettings*)fixtureB->GetUserData();
            b2Vec2 localNormalB = fixtureB->GetBody()->GetLocalVector(-worldManifold.normal);
            tangentialVelocity += conveyorSettings->getTangentialVelocity(localNormalB);
        }
        
        if ( tangentialVelocity != 0 ) {
            tangentialVelocity *= m_currentSpeed;
            contact->SetTangentSpeed(tangentialVelocity);
        }
    }

    void drawSquareTexture(b2Vec2 center, float angleRad, float scale)
    {
        glEnable(GL_BLEND);
        glPushMatrix();
        glTranslatef(center.x, center.y, 0);
        glRotatef(angleRad*RADTODEG, 0,0,1);
        glScalef(scale,scale,1);
        glTranslatef(-0.5f,-0.5f,0);

        glBegin(GL_TRIANGLE_FAN);
        glTexCoord2f(0,0);
        glVertex2f(0,0);
        glTexCoord2f(1,0);
        glVertex2f(1,0);
        glTexCoord2f(1,1);
        glVertex2f(1,1);
        glTexCoord2f(0,1);
        glVertex2f(0,1);
        glEnd();

        glPopMatrix();
    }
    
    void Step(Settings* settings)
    {
        if ( m_tankBody ) {
            float32 desiredViewCenterX = m_tankBody->GetWorldCenter().x;
            settings->viewCenter.x = 0.95f * settings->viewCenter.x + 0.05f * desiredViewCenterX;//avoid 'snapping'
        }
        
        float desiredSpeed = 0;
        switch ( m_keyState ) {
        case k_left: desiredSpeed = m_maxSpeed; break;
        case k_right: desiredSpeed = -m_maxSpeed;  break;
        }
        float speedDiff = desiredSpeed - m_currentSpeed;
        m_currentSpeed += speedDiff * 0.05f;
        m_currentSpeed = b2Clamp(m_currentSpeed, -m_maxSpeed, m_maxSpeed);
        
        Test::Step(settings);
        
        float wheelradius = 0.35f;
        float32 timeStep = settings->hz > 0.0f ? 1.0f / settings->hz : float32(0.0f);
        float distanceTracksMoved = m_currentSpeed * timeStep;
        float revolutions = distanceTracksMoved / ( /*M_2_PI **/ wheelradius );
        float angleChange = revolutions /* * M_2_PI*/;
        m_wheelAngle += angleChange;
        

        float trackLength = 0;
        vector<float> segmentLengths;
        for (int i = 0; i < m_splinePoints.size(); i++) {
            splinePoint& sp0 = m_splinePoints[i];
            splinePoint& sp1 = m_splinePoints[(i+1)%m_splinePoints.size()];
            b2Vec2 pos0 = sp0.getPointNow();
            b2Vec2 pos1 = sp1.getPointNow();
            float len = (pos0 - pos1).Length();
            trackLength += len;
            segmentLengths.push_back(len);
        }
        
        m_firstTrackPointOffset -= distanceTracksMoved;
        if ( m_firstTrackPointOffset < 0 )
            m_firstTrackPointOffset += trackLength;
        else if ( m_firstTrackPointOffset > trackLength )
            m_firstTrackPointOffset -= trackLength;
        
#define numTrackPoints 42
        float trackPointSpacing = trackLength / (float)numTrackPoints;
        int firstTrackPointBaseIndex = 0;
        float rem = m_firstTrackPointOffset;
        float fraction = 0;
        float firstTrackPointBaseSplinePointDistance = 0;
        for (int i = 0; i < segmentLengths.size(); i++) {
            float len = segmentLengths[i];
            if ( rem < len ) {
                firstTrackPointBaseIndex = i;
                fraction = rem / len;
                break;
            }
            rem -= len;
            firstTrackPointBaseSplinePointDistance += len;
        }
        
        b2Vec2 trackPoints[numTrackPoints];
        int sind = firstTrackPointBaseIndex;
        float sdist = m_firstTrackPointOffset - firstTrackPointBaseSplinePointDistance;
        int tind = 0;
        float totalLen = 0;
        while ( tind < numTrackPoints ) {
            float len = segmentLengths[sind];
            totalLen += len;
            if ( sdist < len ) {
                b2Vec2 p0 = m_splinePoints[sind].getPointNow();
                b2Vec2 p1 = m_splinePoints[(sind+1)%m_splinePoints.size()].getPointNow();
                b2Vec2 fpp = p0 + (sdist/len) * (p1 - p0);
                trackPoints[tind++] = fpp;
                sdist += trackPointSpacing;
            }
            else {
                sdist -= len;
                sind++;
                if ( sind >= segmentLengths.size() )
                    sind = 0;
            }
        }
        
        /*int trackPointsRemaining = numTrackPoints;
        //float trackLengthUsed = 0;
        int splinePointIndex = 0;
        float thisSegmentRemaining = 0;
        float basePointLength = 0;
        while ( trackPointsRemaining > 0 ) {
            splinePoint& sp0 = m_splinePoints[splinePointIndex];
            splinePoint& sp1 = m_splinePoints[(splinePointIndex+1)%m_splinePoints.size()];
            b2Vec2 pos0 = sp0.getPointNow();
            b2Vec2 pos1 = sp1.getPointNow();
            float thisSegmentLength = (pos0 - pos1).Length();
            thisSegmentRemaining += thisSegmentLength;
            while ( thisSegmentRemaining >= trackPointSpacing ) {
                //trackLengthUsed += trackPointSpacing;
                trackPointsRemaining--;
                thisSegmentRemaining -= trackPointSpacing;
                float fraction =
            }
            basePointLength += thisSegmentLength;
            splinePointIndex++;
        }*/
        
        /*glPointSize(2);
        glColor3f(0,1,0);
        glBegin(GL_LINES);
        for (int i = 0; i < m_splinePoints.size(); i++) {
            splinePoint& sp0 = m_splinePoints[i];
            splinePoint& sp1 = m_splinePoints[(i+1)%m_splinePoints.size()];
            b2Vec2 pos0 = sp0.getPointNow();
            b2Vec2 pos1 = sp1.getPointNow();
            glVertex2f(pos0.x, pos0.y);
            glVertex2f(pos1.x, pos1.y);
        }
        glEnd();
        glBegin(GL_POINTS);
        for (int i = 0; i < m_splinePoints.size(); i++) {
            splinePoint& sp = m_splinePoints[i];
            b2Vec2 pos = sp.getPointNow();
            glVertex2f(pos.x, pos.y);
        }
        glEnd();*/

        if ( m_drawMode == k_drawFixtures ) {

        }
        else {
            glDisable(GL_DEPTH_TEST);
            glEnable(GL_BLEND);
            glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            glDisable(GL_LIGHTING);
            glColor3f(1,1,1);
            glEnable(GL_TEXTURE_2D);
            glLineWidth(1);

            //draw all textures except for track cover
            for (int i = 0; i < m_textureParams.size()-1; i++) {
                textureParams& tp = m_textureParams[i];
                tp.draw();
            }

            //glDisable(GL_TEXTURE_2D);

            //draw roller wheels
            b2Vec2 upwards = m_tankBody->GetWorldVector( b2Vec2(0,wheelradius*1.1f) );
            for (int i = 0; i < m_wheelguides.size(); i++) {
                b2Vec2 wheelcenter = m_wheelguides[i]->GetPosition() + upwards;
                b2Rot rot(m_wheelAngles[i] + m_wheelAngle);
                b2Vec2 currentWheelAngleVec = b2Mul(rot, b2Vec2(0,1));
                //m_debugDraw.DrawSolidCircle(NULL, wheelcenter, wheelradius, currentWheelAngleVec, b2Color(0.8f,0.8f,0.8f) );
                glBindTexture(GL_TEXTURE_2D, m_rollerWheelTexId);
                drawSquareTexture(wheelcenter, m_wheelAngles[i+7]+m_wheelAngle, 0.75f);
                glBindTexture(GL_TEXTURE_2D, m_wheelShadowTexId);
                drawSquareTexture(wheelcenter-b2Vec2(0,0.05f), 0, 0.75f);
            }
            //draw drive wheels - yes yes you military buffs I know the front one is only a roller so it should have a different texture
            for (int i = 0; i < 1; i++) {
                b2Vec2 wheelcenter = m_endwheels[i]->GetPosition();
                b2Rot rot(m_wheelAngles[i+7] + m_wheelAngle);
                b2Vec2 currentWheelAngleVec = b2Mul(rot, b2Vec2(0,1));
                //m_debugDraw.DrawSolidCircle(NULL, wheelcenter, wheelradius, currentWheelAngleVec, b2Color(0.8f,0.8f,0.8f) );
                glBindTexture(GL_TEXTURE_2D, m_rollerWheelTexId);
                drawSquareTexture(wheelcenter, m_wheelAngles[i+7]+m_wheelAngle, 0.75f);
                glBindTexture(GL_TEXTURE_2D, m_wheelShadowTexId);
                drawSquareTexture(wheelcenter-b2Vec2(0,0.05f), 0, 0.75f);
            }
            //draw front roller - yes yes you military buffs I know the front one is only a roller so it should have a different texture
            for (int i = 1; i < m_endwheels.size(); i++) {
                b2Vec2 wheelcenter = m_endwheels[i]->GetPosition();
                b2Rot rot(m_wheelAngles[i+7] + m_wheelAngle);
                b2Vec2 currentWheelAngleVec = b2Mul(rot, b2Vec2(0,1));
                //m_debugDraw.DrawSolidCircle(NULL, wheelcenter, wheelradius, currentWheelAngleVec, b2Color(0.8f,0.8f,0.8f) );
                glBindTexture(GL_TEXTURE_2D, m_driveWheelTexId);
                drawSquareTexture(wheelcenter, m_wheelAngles[i+7]+m_wheelAngle, 0.9f);
                glBindTexture(GL_TEXTURE_2D, m_wheelShadowTexId);
                drawSquareTexture(wheelcenter-b2Vec2(0,0.05f), 0, 0.9f);
            }

            //draw tracks
            {
                for (int i = 0; i < numTrackPoints; i++) {
                    b2Vec2 fpp0 = trackPoints[i];
                    b2Vec2 fpp1 = trackPoints[(i+1)%numTrackPoints];
                    b2Vec2 axis = fpp1 - fpp0;
                    b2Vec2 mid = 0.5f * (fpp0 + fpp1);
                    float axisLength = axis.Length();
                    axis.Normalize();
                    //fpp0 += 0.05f * axis;
                    //fpp1 -= 0.05f * axis;
                    b2Vec2 cross = b2Cross(1.0f, axis);
                    mid -= (0.5f * axisLength - 0.06f) * cross;
                    b2Vec2 v0 = fpp0 - b2Vec2(0.05f * cross);
                    b2Vec2 v1 = fpp1 - b2Vec2(0.05f * cross);
                    b2Vec2 v2 = fpp1 + b2Vec2(0.05f * cross);
                    b2Vec2 v3 = fpp0 + b2Vec2(0.05f * cross);

                    glColor3f(1,1,1);
                    glBindTexture(GL_TEXTURE_2D, m_trackSegmentTexId);
                    drawSquareTexture(mid, -b2Atan2(-axis.y,axis.x)+180*DEGTORAD, axisLength);

                    /*glColor3f(0,1,0);
                    glBegin(GL_LINES);
                    glVertex2f(v0.x, v0.y);
                    glVertex2f(v1.x, v1.y);
                    glVertex2f(v1.x, v1.y);
                    glVertex2f(v2.x, v2.y);
                    glVertex2f(v2.x, v2.y);
                    glVertex2f(v3.x, v3.y);
                    glVertex2f(v3.x, v3.y);
                    glVertex2f(v0.x, v0.y);
                    glEnd();*/
                }

                /*for (int i = 0; i < numTrackPoints; i++) {
                    b2Vec2 fpp0 = trackPoints[i];
                    m_debugDraw.DrawSolidCircle(NULL, fpp0, 0.05, b2Vec2(0,0), b2Color(0.8f,0.8f,0.8f) );
                }*/
            }

            //draw track cover
            glEnable(GL_TEXTURE_2D);
            glEnable(GL_BLEND);
            if ( m_drawMode == k_drawTextureCoverTracks )
                m_textureParams[m_textureParams.size()-1].draw();

            glDisable(GL_TEXTURE_2D);
        }

        m_debugDraw.DrawString(5, m_textLine, "Press a/d to drive the tank");
        m_textLine += 15;
        m_debugDraw.DrawString(5, m_textLine, "Draw mode (press t to change): %s", m_drawMode == k_drawFixtures?"fixtures":m_drawMode==k_drawTextureShowTracks?"textures (show tracks)":"textures (cover tracks)");
        m_textLine += 15;
        m_debugDraw.DrawString(5, m_textLine, "Current speed: %f", m_currentSpeed);
        m_textLine += 15;
        m_debugDraw.DrawString(5, m_textLine, "Track length: %f", trackLength);
        m_textLine += 15;
    }
    
    static Test* Create()
    {
        return new Tank;
    }
    
    vector<ConveyorSettings*> m_conveyorSettingsToDelete;
    int m_keyState;
    b2Body* m_tankBody;
    float m_maxSpeed;
    float m_currentSpeed;
    vector<b2Body*> m_wheelguides;//7
    vector<b2Body*> m_endwheels;//2
    float m_wheelAngle;
    float m_wheelAngles[9];
    
    vector<splinePoint> m_splinePoints;
    float m_firstTrackPointOffset;
    
    vector<textureParams> m_textureParams;

    TankDebugDraw* m_tankDebugDraw;
    int m_drawMode;
    set<b2Body*> m_noDrawBodyList;

    GLuint m_driveWheelTexId;
    GLuint m_rollerWheelTexId;
    GLuint m_wheelShadowTexId;
    GLuint m_trackSegmentTexId;
};

#endif


